///////////////////////////////////////////////////////////////////////////////
//
// StlAllocatorTestMain.h
//
// Copyright  2003 Pete Isensee (PKIsensee@msn.com).
// All rights reserved worldwide.
//
// Permission to copy, modify, reproduce or redistribute this source code is
// granted provided the above copyright notice is retained in the resulting 
// source code.
// 
// This software is provided "as is" and without any express or implied
// warranties.
//
// Description:
//
//    This module is designed for testing custom allocators. Simply pass your
//    custom allocator to the TestStlAllocator() function, stand back and let
//    the show begin.
//
// Details:
//
//    TestStlAllocator() verifies that the incoming allocator meets the
//    requirements of the C++ Standard. It does this in three ways:
// 
//    1) Ensuring that all allocator functions are available. If a 
//       function is not available, the code will fail to compile.
//    2) Ensuring that all allocator functions work as advertised. 
//       If the functions fail, error messages are sent to standard ouput.
//    3) Ensuring that the allocator works properly will all standard
//       containers using a variety of stored objects.

#if !defined( STL_ALLOCATOR_TEST_MAIN_H )
#define STL_ALLOCATOR_TEST_MAIN_H
#pragma once

#pragma warning( disable: 4127 ) // ignore conditional expression constant
#pragma warning( disable: 4702 ) // ignore unreachable code
#pragma warning( disable: 4800 ) // allow forcing int to bool

#include "StlAllocatorTestTypes.h"
#include "StlAllocatorTestMembers.h"
#include "StlAllocatorTestContainer.h"

namespace StlAllocatorTestbed
{

//-----------------------------------------------------------------------------

template< typename Allocator >
void TestAlloc( const Allocator& a )
{
    // Verify functionality with a variety of types
    Allocator::rebind< bool >::other   b( a ); // tests vector<bool> specialization
    Allocator::rebind< C >::other      c( a ); // POD types
    Allocator::rebind< D >::other      d( a ); // allocated internal type
    Allocator::rebind< E >::other      e( a ); // complicated type
    Allocator::rebind< int >::other    i( a ); // simple test
    Allocator::rebind< void* >::other  v( a ); // void pointers
    Allocator::rebind< char* >::other  p( a ); // regular pointers
    Allocator::rebind< void >::other   x( a ); // must provide this specialization

    TestMemberFunctionPresence( a ); // in StlAllocatorTestMembers.h
    
    TestMemberFunctions( a ); // in StlAllocatorTestMembers.h
    TestMemberFunctions( b );
    TestMemberFunctions( c );
    TestMemberFunctions( d );
    TestMemberFunctions( e );
    TestMemberFunctions( i );
    TestMemberFunctions( v );
    TestMemberFunctions( p );
    
    TestWithContainers( a ); // in StlAllocatorTestContainer.h
    TestWithContainers( b );
    TestWithContainers( c );
    TestWithContainers( d );
    TestWithContainers( e );
    TestWithContainers( i );
    TestWithContainers( v );
    TestWithContainers( p );
}

} // end StlAllocatorTestbed namespace

#endif // STL_ALLOCATOR_TEST_MAIN_H
